#!/usr/bin/perl  
#
# Copyright (C) 2011 Generalitat de Catalunya 
# 
# This file is part of Linkatedu_11_4.  
# 
# Linkatedu_11_4 is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.
# 
# Linkatedu_11_4 is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for further
# details, currently published at http://www.gnu.org/licenses/gpl.txt or in the
# gpl.txt in the ["legal" folder] of this distribution.
# 
# You should have received a copy of the GNU General Public License along with
# this program; If not, see <http://www.gnu.org/licenses/>
# 
# This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
# 
# Jordi "Txor" Casas Ríos <jcasas@opentrends.net>, openTrends Solucions i Sistemes, S.L.

use Tk;
use Tk::NoteBook;
use Tk::HList; 
use LTSP_utils::ConfigList;
use File::Copy;
use strict;
use warnings;
use utf8;

######
# Main code

# Create the objects
#TODO: Get the IP by a more elegant way.
my $ip = ` grep SERVER= /srv/tftpboot/KIWI/lts.conf | cut -f 2 -d '=' `;
chomp($ip);
#TODO: Don't use absolute paths, use configuration files and sysconfig...
my $cure_path = '/usr/share/linkatedu_11_4-LTSP-fix-icons_intel/icons_intel-cure';
&installCure();
my @cure;
push(@cure, [ '/KIWI/50-device.conf', '/etc/X11/xorg.conf.d/50-device.conf', $ip ]);
push(@cure, [ '/KIWI/intellegacy_drv.la', '/usr/lib/xorg/modules/drivers/intellegacy_drv.la', $ip ]);
push(@cure, [ '/KIWI/intellegacy_drv.so', '/usr/lib/xorg/modules/drivers/intellegacy_drv.so', $ip ]);
my $LTSPConfigList = LTSP_utils::ConfigList->new();
my $tl;
my $magic_label;
my $mw = MainWindow->new;

# Create the tab 
my $book = $mw->NoteBook()->pack( -fill=>'both', -expand=>1 );
my $tab1 = $book->add( "Sheet 1", -label=>"Areglar problema de icones amb intellegacy" );

# Create the content.
# Frame 1 as list container.
my $frame1 = $tab1->Frame()->pack( -fill => 'both', -expand => 1 );
my $hlist = $frame1->HList( -columns=>2, -header => 1, -selectmode => 'single'); 
$hlist->header('create', 0, -itemtype => 'text', -text => "Arreglar") ; 
$hlist->header('create', 1, -itemtype => 'text', -text => "Fitxers de configuració") ; 
my $scroll = $frame1->Scrollbar(-command => ['yview', $hlist]);
$hlist->configure( -yscrollcommand => ['set', $scroll] );
$hlist->pack( -side => 'left', -fill => 'both', -expand => 1 );
$scroll->pack( -side => 'right', -fill => 'y' );

# Populate the hlist
#&pushItem($hlist, 'config.default');
my $path = $LTSPConfigList->path();
foreach ($LTSPConfigList->listConfig()) {
	chomp;
	$_ =~ s/$path\/(.*)/$1/;
	&pushItem($hlist, $_) unless $_ eq 'config.default';
}

# Frame 2 as button container
my $frame2 = $tab1->Frame()->pack( -fill=>'x' );
$frame2->Button( -text=>'Nou fitxer', -command=>\&novaConfig )->pack( -side => 'left' );
$frame2->Button( -text=>'Esborra fitxer', -command=>\&borraConfig )->pack( -side => 'left' );
$frame2->Button( -text=>'Surt', -command=>sub{ exit; } )->pack( -side => 'right' );

&center($mw);
MainLoop;

######
# Subroutines
sub center {
  my $win = shift;

  $win->withdraw;   # Hide the window while we move it about
  $win->update;     # Make sure width and height are current

  # Center window
  my $xpos = int(($win->screenwidth  - $win->width ) / 2);
  my $ypos = int(($win->screenheight - $win->height) / 2);
  $win->geometry("+$xpos+$ypos");

  $win->deiconify;  # Show the window again
}

# pushItem creates a new entry with the asociated new objects.
sub pushItem {
	my $container = shift;
	my $name = shift;
	my $e = $container->addchild( "" ); 
	my $cb = $container->Checkbutton();
	$hlist->itemCreate( $e, 0, -itemtype=>'window', -window => $cb ); 
	$hlist->itemCreate( $e, 1, -itemtype=>'text', -text=>$name ); 
	$cb->configure( -command => [ \&modificarEstatConfig, $name, $cb ] );
	if ($LTSPConfigList->queryConfig($name, $cure[0][0], $cure[0][1], $cure[0][2]) ne 0 and $LTSPConfigList->queryConfig($name, $cure[1][0], $cure[1][1], $cure[1][2]) ne 0 and $LTSPConfigList->queryConfig($name, $cure[2][0], $cure[2][1], $cure[2][2]) ne 0) { $cb->{'Value'} = 1; }
}

sub novaConfig {
	if (! Exists($tl)) {
		$tl = $mw->Toplevel;
		$tl->title('Afegir configuració per MAC address');
		$magic_label = $tl->Label(-text => "Introduïu una adreça MAC.")->pack( -side => 'top' );
		my $mac_entry = $tl->Entry()->pack( -side => 'top',  -expand => 1 );
		$tl->Button( -text => 'Afegeix', -command => [ \&novaConfig_insertElement, $mac_entry ] )->pack( -side => 'left' );
		$tl->Button( -text => 'Cancela', -command => sub { $tl->destroy; } )->pack( -side => 'right' );
		$mac_entry->focusForce;
	}
	else {
		$tl->deiconify;
		$tl->raise;
	}
	&center($tl);
}

sub novaConfig_insertElement {
	my $entry = shift;
	my $text = $entry->get;
	if ($text =~ m/([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}/) {
		&pushItem($hlist, 'config.'.$text) unless $LTSPConfigList->addConfig('config.'.$text) == 0;
		$tl->destroy;
	}
	else {
		$magic_label->configure(-text => "Introduïu una adreça MAC correcta!");
	}
}

sub borraConfig {
	my $sel = $hlist->info('selection');
	return 0 unless defined $sel;
	my $name = $hlist->itemCget( $hlist->selectionGet, 1, 'text' );
	return 0 unless $name ne 'config.default';
	$LTSPConfigList->delConfig($name);
	$hlist->delete('entry', $sel);
}

sub modificarEstatConfig {
	my $name = shift;
	my $cb = shift;
	if ($cb->{'Value'} == 0) {
		$LTSPConfigList->modifyConfig($name, 'DEL', $cure[0][0], $cure[0][1], $cure[0][2]);
		$LTSPConfigList->modifyConfig($name, 'DEL', $cure[1][0], $cure[1][1], $cure[1][2]);
		$LTSPConfigList->modifyConfig($name, 'DEL', $cure[2][0], $cure[2][1], $cure[2][2]);
	}
	else {
		$LTSPConfigList->modifyConfig($name, 'ADD', $cure[0][0], $cure[0][1], $cure[0][2]);
		$LTSPConfigList->modifyConfig($name, 'ADD', $cure[1][0], $cure[1][1], $cure[1][2]);
		$LTSPConfigList->modifyConfig($name, 'ADD', $cure[2][0], $cure[2][1], $cure[2][2]);
	}
}

sub installCure {
	copy($cure_path.'/50-device.conf', '/srv/tftpboot/KIWI/50-device.conf') or die $! unless -f '/srv/tftpboot/KIWI/50-device.conf';
	copy($cure_path.'/intellegacy_drv.la', '/srv/tftpboot/KIWI/intellegacy_drv.la') or die $! unless -f '/srv/tftpboot/KIWI/intellegacy_drv.la';
	copy($cure_path.'/intellegacy_drv.so', '/srv/tftpboot/KIWI/intellegacy_drv.so') or die $! unless -f '/srv/tftpboot/KIWI/intellegacy_drv.so';
}

